package xena;

import java.util.ArrayList;

public class ProtocolHandler {

	public ProtocolHandler(ProtocolType type) {
		super();
		this.type = type;
	}

	ArrayList<ProtocolField> fields = new ArrayList<ProtocolField>();
	ProtocolType type;
	
	public void setValue(String field,int value)
	{
		for (ProtocolField f : fields)
		{
			if (field.compareToIgnoreCase(f.name)==0)
			{
				f.setValue(value);
				return;
			}
		}
		
	}
	public void AddField(String name,int length)
	{
		AddField(name,length,0);
	}
	
	public void AddField(String name,int length,int value)
	{
		ProtocolField field = new ProtocolField(name,length,value);
		fields.add(field);
	}
	
	@Override
	public String toString() {
		StringBuilder binValue = new StringBuilder();
		for (ProtocolField field : fields)
		{
			binValue.append(field.toString());
		}
		// convert the Binary to hex
		return binToHex(binValue.toString());
	}

	private String binToHex(String bin)
	{
		StringBuilder ret = new StringBuilder(bin.length()/4+1);
		for(int i=0;i<bin.length();i+=4)
		{
			int count = Math.min(4, bin.length()-i);
			int value = 0;
			for (int j=0;j<count;j++)
			{
				value = value << 1;
				if (bin.charAt(i+j)=='1')
					value +=1;
			}
			ret.append(Integer.toHexString(value));
		}
		return ret.toString();
	}
	
	public class ProtocolField
	{
		public ProtocolField(String name, int length) {
			this(name,length,0);
		}
		
		public ProtocolField(String name,int length,int value) 
		{
			super();
			this.name = name;
			this.length = length;
			this.value = value;
		}
		
		public String getName() {
			return name;
		}
		public int getLength() {
			return length;
		}
		
		public int getValue() {
			return value;
		}

		public void setValue(int value) {
			this.value = value;
		}
		
		
		@Override
		public String toString() {
			// convert to bit string
			StringBuilder ret = new StringBuilder(length);
			for (int i=(length-1);i>=0;i--)
			{
				boolean bit = ((value>>i) & 1) == 1;
				ret.append(bit ? "1" : "0");
			}
			return ret.toString();
		}


		final String name;
		final int length; // in bits
		int value;
		
	}
	
	public enum ProtocolType
	{
		ETHERNET(14),
		VLAN(4),
		ARP(28),
		IP(20),
		IPV6(40),
		UDP(8),
		TCP(20),
		LLC(3),
		SNAP(5),
		GTP(20),
		ICMP(8),
		RTP(12),
		RTCP(4),
		STP(35),
		SCTP(12),
		MACCTRL(4),
		MPLS(4),
		PBBTAG(4),
		FCOE(14),
		FC(24),
		FCOETAIL(4),
		IGMP0(12),
		IGMP1(16);
		
		int length; 
		public int getLength() {
			return length;
		}
		
		ProtocolType(int len)
		{
			length = len;
		}
		
	}
	
	public static ProtocolHandler getCVlan()
	{
		ProtocolHandler prot = new ProtocolHandler(ProtocolType.VLAN);
		prot.AddField("TPID", 16,0x8100);
		prot.AddField("PCP", 3);
		prot.AddField("CFI", 1);
		prot.AddField("VID", 12);
		return prot;
	}
}
