package xena;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.ArrayList;

import ethernetPacket.EthernetPacket;

import xena.XenaAPI;
import xena.XenaCom.PC_EXTRA_RESPONSE;
import xena.XenaCom.XenaLatencyStatistics;
import xena.XenaCom.XenaStatistics;
import xena.XenaCom.portCaptureKeepCriteria;
import xena.XenaCom.portCaptureStartStopCriteria;
import xena.XenaPacket.packetSizeTypes;

import jsystem.framework.system.SystemObjectImpl;

public class Xena extends SystemObjectImpl implements XenaAPI {
	String ip;
	String password;
	String user;
	boolean debug;
	boolean useRelinquish;
	boolean xenaEnabled;
	int communicationSleep_msec = 2;
	public int getCommunicationSleep_msec() {
		return communicationSleep_msec;
	}
	public void setCommunicationSleep_msec(int communicationSleep_sec) {
		this.communicationSleep_msec = communicationSleep_sec;
	}

	String capturePathForThread="";
	ArrayList <saveCapturedTraffic> Traffic= new ArrayList <saveCapturedTraffic>();

	XenaPort[] ports;
	XenaCom xena = new XenaCom();
	ArrayList<XenaStream> streams = new ArrayList<XenaStream>();
	private int captureDuration_ms = 1000;

	protected XenaPort getPortParams(int portLogicalAddress) {
		if (portLogicalAddress >= ports.length)
			return null;
		else
			return ports[portLogicalAddress];
	}
	public int getModule(int portLogicalAddress) {
		if (getPortParams(portLogicalAddress) == null)
				return -1;
		else
			return getPortParams(portLogicalAddress).module;
	}
	
	public int getPort(int portLogicalAddress) {
		if (getPortParams(portLogicalAddress) == null)
				return -1;
		else
			return getPortParams(portLogicalAddress).port;
	}
	
	public boolean isUseRelinquish() {
		return useRelinquish;
	}


	public void setUseRelinquish(boolean useRelinquish) {
		this.useRelinquish = useRelinquish;
	}
	
	public boolean isXenaEnabled() {
		return xenaEnabled;
	}


	public void setXenaEnabled(boolean xenaEnabled) {
		this.xenaEnabled = xenaEnabled;
	}


	public String getIp() {
		return ip;
	}
	 
	@Override
	public void close() 
	{
		try {
			report.report("Closing Xena connection");
			stop();
			for (int i = 0; i < ports.length; i++) {
				xena.P_RELEASE(ports[i].module, ports[i].port);
			}
			xena.C_LOGOFF();
			xena.C_DISCONNECT();
		} catch (Exception e) {
		}
		super.close();
		
	}

	public  void setIp(String ip) {
		this.ip = ip;
	}
	public String getPassword() {
		return password;
	}
	public void setPassword(String password) {
		this.password = password;
	}
	public String getUser() {
		return user;
	}
	public void setUser(String user) {
		this.user = user;
	}

	protected void debug(String msg)
	{
		if (debug)
			report.report(msg);
		else
			System.out.println(msg);
	}

	public mps[] moduleAndPort;
	private long defaultPacketCaptureLength = 10; // by default capture 10 packets only, to save processing time
	public long getDefaultPacketCaptureLength() {
		return defaultPacketCaptureLength;
	}
	public void setDefaultPacketCaptureLength(long defaultPacketCaptureLength) {
		this.defaultPacketCaptureLength = defaultPacketCaptureLength;
	}

	private long packetCaptureLength;
	private int captureBytesToKeep = -1;
	public int getCaptureBytesToKeep() {
		return captureBytesToKeep;
	}
	public void setCaptureBytesToKeep(int captureBytesToKeep) {
		this.captureBytesToKeep = captureBytesToKeep;
	}
	public long getPacketCaptureLength() {
		return packetCaptureLength;
	}
	public void setPacketCaptureLength(long packetCaptureLength) {
		this.packetCaptureLength = packetCaptureLength;
	}
	public void setModuleAndPort(mps[] _moduleAndPort) {
		this.moduleAndPort = _moduleAndPort;		
	}

	@Override 
	public void init() throws Exception {
		super.init();
		
		packetCaptureLength = defaultPacketCaptureLength;
		ports = new XenaPort[moduleAndPort.length];
		// Iterate all port numbers 
		for (int iterator = 0; iterator < ports.length; iterator++) {
			ports[iterator] = new XenaPort(moduleAndPort[iterator].port, moduleAndPort[iterator].module);
		}
		System.out.println("xenaEnabled :"+xenaEnabled);
		if (xenaEnabled){
		xena.setOwner(this);
		xena.C_CONNECT(ip);
		debug("Connected to Xena at " + ip);
		xena.C_LOGON(password);	
		xena.C_OWNER(this.user);
		for (int i=0;i<ports.length;i++)
		{
			xena.P_RESERVE_SAFE(ports[i].module, ports[i].port,useRelinquish);
			Thread.sleep(1000); // give the ports time to go up again.
			xena.P_RESET(ports[i].module, ports[i].port);
		}
	//	boolean mine = xena.P_IS_RESERVE_BY_ME(module, port0);
		Thread.sleep(5000); // give the ports time to go up again.
	}
	}
//

	protected  void reconnect() throws Exception {
		report.report("Reconnecting to Xena");
		xena.C_CONNECT(ip);
		xena.C_LOGON(password);	
		xena.C_OWNER(user);
		for (int i=0;i<ports.length;i++)
		{
			xena.P_RESERVE_SAFE(ports[i].module, ports[i].port,true);
		}
		Thread.sleep(5000); // give the ports time to go up again.
	}

	@Override
	public void reset()  throws Exception
	{
		streams.removeAll(streams);
		XenaStream.ResetStreamNumber();
		for (int i=0;i<ports.length;i++)
		{
			report.report(String.format("Resetting module %d / port %d!", ports[i].module, ports[i].port));
			xena.P_RESET(ports[i].module, ports[i].port);
		}
	}

	@Override
	public void removeAllStreams() throws Exception
	{
		report.report("Clearing all of Xena's streams");
		streams.removeAll(streams);
		for (int i=0;i<ports.length;i++)
		{
			xena.P_RESET(ports[i].module, ports[i].port);
		}
		report.report("Sleeping 10 sec");
		// Reset the stream ID counter
		XenaStream.ResetStreamNumber();
		Thread.sleep(10000);
	}
	
	
	public  XenaStream AddStream(int port,String name,EthernetPacket packet) throws Exception
	{
		int _module = ports[port].module;
		int _port = ports[port].port;
		streams.add(new XenaStream(this,port,_module,_port,name,packet, packetSizeTypes.Fixed, 64, 64, new streamBurstParams(0, 100), -1));

		return getStream(streams.size()-1);

	}

	@Override
	public XenaStream getStream(int index)
	{
		return streams.get(index);
	}

	@Override
	public void start(int msec) throws Exception
	{
		start(msec, false, null);
	}
	
	@Override
	public void start(int msec, boolean captureTraffic, String capturePath) throws Exception {	
		
		report.report(String.format("Starting traffic for %.2f seconds, Capture %s", ((double)msec) / 1000, captureTraffic));
		if (captureTraffic) {
			for (int portIterator = 0; portIterator < ports.length; portIterator++) {
				preparePortForCapture(portIterator);
			}
		}
		if (captureTraffic) {
			for (int portIterator = 0; portIterator < ports.length; portIterator++) {
				xena.P_CAPTURE(ports[portIterator].module, ports[portIterator].port, true);
				
					Traffic.add(new saveCapturedTraffic(xena, ports, packetCaptureLength,portIterator, String.format("%s_port%d.pcap", capturePath, portIterator)));
					Traffic.get(portIterator).start();
				
			}
		}
		start();
		double startTime = java.lang.System.nanoTime() / Math.pow(10, 6);
		
		Thread.sleep((long) (msec - (java.lang.System.nanoTime()/Math.pow(10, 6) - startTime)));
		stop();
		Thread.sleep(4000);
		if (captureTraffic) {
			report.report("Saving captured packets to file...");
			for (int portIterator = 0; portIterator < ports.length; portIterator++) {
				xena.P_CAPTURE(ports[portIterator].module, ports[portIterator].port, false);
				//saveCapturedTrafficToFile(portIterator, String.format("%s_port%d.pcap", capturePath, portIterator));
			}
		}

	}
	
	
	protected void apply() throws Exception
	{
		for (int i=0;i<streams.size();i++)
		{
			streams.get(i).apply();
			streams.get(i).enable(true); // make sure stream is enabled
		}
	}
	@Override
	public void setCapturePathForThread(String capturePath){
		capturePathForThread=capturePath;
		
	}
	@Override
	public void start(Boolean captureTraffic ) throws Exception
	{
		Thread.sleep(2000);
		debug("Xena start traffic");
		apply();

		for (XenaStream stream : streams)
		{
			stream.enable(stream.getState().enabled);
		}
		
		if (captureTraffic) {
			for (int portIterator = 0; portIterator < ports.length; portIterator++) {
				preparePortForCapture(portIterator);
			}
		}

//		for (int i=0;i<ports.length;i++)
//		{
//			if (xena.PS_COUNT(ports[i].module, ports[i].port) > 0) {
//				report.report(String.format("Started traffic on module %d / port %d!", ports[i].module, ports[i].port));
//				xena.P_TRAFFIC(ports[i].module, ports[i].port,true);
//			}
//			
//		}	
		if (captureTraffic) {
			for (int portIterator = 0; portIterator < ports.length; portIterator++) {
				xena.P_CAPTURE(ports[portIterator].module, ports[portIterator].port, true);
							
			}
		for (int portIterator = 0; portIterator < ports.length; portIterator++) {
			Traffic.add(new saveCapturedTraffic(xena, ports, packetCaptureLength,portIterator, String.format("%s_port%d.pcap", capturePathForThread, portIterator)));
			(Traffic.get(portIterator)).start();
			}
		}
		for (int i=0;i<ports.length;i++)
		{
			if (xena.PS_COUNT(ports[i].module, ports[i].port) > 0) {
				report.report(String.format("Started traffic on module %d / port %d!", ports[i].module, ports[i].port));
				xena.P_TRAFFIC(ports[i].module, ports[i].port,true);
			}
			
		}	
		
		
	}
	
	//yair
	@Override
	public void start1(Boolean captureTraffic ) throws Exception
	{
		Thread.sleep(2000);
		debug("Xena start traffic");
		//apply();

			for (int portIterator = 0; portIterator < ports.length; portIterator++) {
				preparePortForCapture(portIterator);
			}

			for (int portIterator = 0; portIterator < ports.length; portIterator++) {
				xena.P_CAPTURE(ports[portIterator].module, ports[portIterator].port, true);
							
			}
		for (int portIterator = 0; portIterator < ports.length; portIterator++) {
			Traffic.add(new saveCapturedTraffic(xena, ports, packetCaptureLength,portIterator, String.format("%s_port%d.pcap", capturePathForThread, portIterator)));
			(Traffic.get(portIterator)).start();
			}
		}
		
	
	
	
	@Override
	public void stop1(Boolean captureTraffic, String capturePath) throws Exception
	{
		Thread.sleep(2000);
		debug("Xena stop traffic and capture (in playback mode)");
		
		
		stop();	
		report.report("Saving captured packets to file...(in playback mode)");
		for (int portIterator = 0; portIterator < ports.length; portIterator++) {
		xena.P_CAPTURE(ports[portIterator].module, ports[portIterator].port, false);
			//saveCapturedTrafficToFile(portIterator, String.format("%s_port%d.pcap", capturePath, portIterator));
		}
		for(int i=0;i<Traffic.size();i++)
			Traffic.get(i).join();
		Traffic.clear();
		
	}
		
	
	//end yair

	@Override
	public void stop(Boolean captureTraffic, String capturePath) throws Exception
	{
		
		debug("Xena stop traffic and capture");
		
		stop();
		
	if (captureTraffic) {
		
		report.report("Saving captured packets to file...");
		for (int portIterator = 0; portIterator < ports.length; portIterator++) {
		xena.P_CAPTURE(ports[portIterator].module, ports[portIterator].port, false);
			//saveCapturedTrafficToFile(portIterator, String.format("%s_port%d.pcap", capturePath, portIterator));
		}
		for(int i=0;i<Traffic.size();i++)
			Traffic.get(i).join();
		Traffic.clear();
		
		}
	
		
	}
	@Override
	public void stop() throws Exception
	{
		
		debug("Xena stop traffic");
		
		
		for (int i=0;i<ports.length;i++)
		{
			if (xena.PS_COUNT(ports[i].module, ports[i].port) > 0) {
				report.report(String.format("Stopped traffic on module %d / port %d!", ports[i].module, ports[i].port));
				xena.P_TRAFFIC(ports[i].module, ports[i].port,false);
			}
		}
		
		Thread.sleep(5000); // wait a little after stop to make sure traffic stopped and line are clear we got all packets	
		
		saveCapturedTraffic.stop=false;
	}
	
	@Override
	public XenaStatistics getPortTxStatistcs(int port) throws Exception
	{
		return xena.PT_TOTAL(ports[port].module, ports[port].port);
	}
	
	@Override
	public XenaStatistics getPortRxStatistcs(int port) throws Exception
	{
		return xena.PR_TOTAL(ports[port].module, ports[port].port);
	}

	@Override
	public void clearPortStatistics(int port) throws Exception
	{
		xena.P_CLEAR(ports[port].module, ports[port].port);
	}

	@Override
	public void clearAllStatistics() throws Exception
	{
		for (int i=0;i<ports.length;i++)
		{
			clearPortStatistics(i);
		}
		Thread.sleep(1000);
	}


	protected XenaStatistics getStreamRxStatistics(int port,int sid) throws Exception
	{
		return xena.PR_STREAM(ports[port].module, ports[port].port,sid);
	}

	public XenaStatistics getStreamTxStatistcs(int port,int sid) throws Exception
	{
		return xena.PT_STREAM(ports[port].module, ports[port].port,sid);
	}
	
	public XenaStatistics getTxStatistics(int sid) throws Exception
	{
		XenaStatistics stat = null;
		for (int i=0;i<ports.length;i++)
		{
			stat = getStreamTxStatistcs(i, sid);
			if (stat.packets != 0) {
				stat.setReceivedPortNumber(i);
				return stat;
			}
		}
		return stat;
	
	}
	/*

	 * 
	 */
	@Override
	public XenaStatistics getRxStatistics(int sid) throws Exception
	{
		XenaStatistics stat = null;
		for (int i=0;i<ports.length;i++)
		{
			stat = getStreamRxStatistics(i, sid);
			if (stat.packets != 0) {
				stat.setReceivedPortNumber(i);
				return stat;
			}
		}
		return stat;
	
	}

	@Override
	public XenaLatencyStatistics getRxLatencyStatistics(int sid) throws Exception
	{
		XenaLatencyStatistics stat = null;
		for (int i=0;i<ports.length;i++)
		{
			stat = xena.PR_STREAM_LATENCY(ports[i].module, ports[i].port, sid);
			stat.jitter = xena.PR_TPLDJITTER(ports[i].module, ports[i].port, sid);
			if (stat.min > 0)
				return stat;
		}
		return stat;
	
	}

	@Override
	public XenaLatencyStatistics getLatencyStatistics() throws Exception
	{
		XenaLatencyStatistics stat = streams.get(0).getLatencyStatistics();
		for (int i=1;i<streams.size();i++)
		{
			stat = stat.merge(streams.get(i).getLatencyStatistics());
		}
		return stat;
		
	}
	
	@Override
	public portConfigurationResponse configurePort(portConfigurationParams params) throws Exception {
		portConfigurationResponse response = new portConfigurationResponse();
		
		// AutoNeg selection
		ports[params.portLogicalAddress].setAutoneg(params.autoNeg);
		
		// Set the rate (in mbps)
		ports[params.portLogicalAddress].setSpeed(params.rate.getRateMbps());
		
		
		// Set the description field 
		ports[params.portLogicalAddress].setPortComment();
		
		// Read all important parameters back into the response
		response.syncStatus = xena.P_SYNC(ports[params.portLogicalAddress].module, ports[params.portLogicalAddress].port);
		response.rate = portRates.toRate(ports[params.portLogicalAddress].getSpeed()); 
		response.setSuccess(true);
		
		return response;
	}

	/* (non-Javadoc)
	 * @see xena.XenaAPI#createStream(xena.XenaAPI.streamCreationParams)
	 */
	@Override
	public streamCreationResponse createStream(streamCreationParams params) throws Exception {
		streamCreationResponse response = new streamCreationResponse();
		response.streamID = 0;
		
		// This method creates a stream and performs the following configurations:
		// 0. Create a new XenaStream object
		// 1. Sets the rate of the stream
		// 2. Creates a new packet for the stream
		// 3. Adds the relevant headers requested by the user
		// 4. Adds the modifiers to the stream
		// 5. Sets the burst size
		// 6. Sets the stream's state
		// 7. Saves the stream in the Xena object
		
		XenaPort currentPort = ports[params.portLogicalAddress];
		EthernetPacket thePacket = new EthernetPacket();

		// 0.
		thePacket.setName(params.packet.packetName);
		// 3
		thePacket.setHeaders(params.packet.header);

		XenaStream newStream = new XenaStream(this,params.portLogicalAddress, currentPort.module, currentPort.port, 
				params.streamName, thePacket, params.packet.packetSize, params.packet.packetSizeMin, 
				params.packet.packetSizeMax, params.getBurst(), params.packet.stopAfterNumOfPackets);
		// 1.
		
		//Convert configured rate to Mbps for fixed packet size
		if ( params.packet.packetSize.patternString.equals("FIXED") && !params.rate.type.description.equals("Mbps") ){
			report.report(String.format("Convert %.2f %s To Mbps", params.rate.value, params.rate.type));
			if ( params.rate.type.description.equals("Percent") ){
				double percentageToMbpsConversion = ( params.rate.value * params.packet.packetSizeMin * 10.0) / ( 20.0 + params.packet.packetSizeMin ); 
				params.setRate(streamRateTypes.Mbps, percentageToMbpsConversion);
			}
			if (params.rate.type.description.equals("PPS")){
				double ppsToMbpsConversion = params.rate.value * params.packet.packetSizeMin * 8.0 / 1000000.0; 
				params.setRate(streamRateTypes.Mbps, ppsToMbpsConversion);
			}
		}
		//Check total configured rate is no more than 1Gbps per port
		double totalRate = (params.getRate().getValue() / params.packet.packetSizeMin) * (20 + params.packet.packetSizeMin);
		for (XenaStream stream : streams){
			if (stream.get_port() == currentPort.port && stream.get_module() == currentPort.module)
				totalRate += ( (double) stream.getRateMbps() / stream.getMinPacketSize()) * (20 + stream.getMinPacketSize());
		}
		if ( totalRate > 1000 )
			throw new Exception(String.format("Total configured rate is more than 100%%"));
			

		newStream.setRate(params.rate);
			
		// 3.
		if (!params.payloadPattern.startsWith("0x"))
			params.payloadPattern = String.format("0x%s", params.payloadPattern);
		
		newStream.setPayloadTypeAndPattern(params.payloadType, params.payloadPattern);
		// 4.
		
		// 5.
		newStream.setBurst(params.burst);

		// 6.
		newStream.setState(params.state);
		// 7.
		streams.add(newStream);
		
		response.streamID = newStream.getSid();
//		report.report(String.format("Successfully created a stream on module %d / port %d: stream ID = %d", currentPort.getModule(), currentPort.getPort(), response.streamID));
		report.report(String.format("Stream ID %d (m%d/p%d): Rate=%.2f () %s PacketSize=%d Pattern=%s State=%s", response.streamID,currentPort.getModule(), currentPort.getPort(),params.rate.getValue(),params.rate.type.toString(),params.packet.packetSizeMin,params.packet.packetSize.toString(),params.state.toString()),thePacket.toString(),true);
		response.success = true;		
		return response;
	}

	
	@Override
	public void injectErrorsIntoStreamCheckSum(int streamID, int N) throws Exception {
		for (XenaStream currentStream : streams) {
			if (currentStream.getSid() == streamID) {
				for(int i=1;i<=N;i++){
				currentStream.setCheckSumError();
				}
				report.report(String.format("Injected FCS errors on stream %d - total %d for this stream", streamID, currentStream.getNumberOfInjectedFcsErrors()));
			return;	
			}
		}
		report.report(String.format("Error injection failed since there is no stream defined with sid = %d", streamID), false);
		return;
	}
	
	@Override
	public void injectErrorsIntoStreamTPLD(/*int logicalPortAddress, */int streamID, int N) throws Exception {
		for (XenaStream currentStream : streams) {
			if (currentStream.getSid() == streamID) {
				for(int i=1;i<=N;i++){
				currentStream.setTPLDError();
				}
				
				report.report(String.format("Injected TPLD errors on stream %d - total %d for this stream", streamID, currentStream.getNumberOfInjectedTPLDErrors()));
				return;	
			}
		}
		report.report(String.format("Error injection failed since there is no stream defined with sid = %d", streamID), false);
		return;	
	}
	
	@Override
	public boolean setStreamState(int logicalPortAddress, int streamID, streamStates state) throws Exception {
		
		XenaPort thePort = this.ports[logicalPortAddress];

		for (XenaStream stream : this.streams) {
			if ((stream.getSid() == streamID) && (stream.get_module() == thePort.getModule()) && (stream.get_port() == thePort.getPort())) {
				stream.setState(state);
				return true;
			}
		}
		// If we reached this point then the user requested we change the state of a stream whose parameters cannot be found!
		throw new Exception(String.format("Unable to find a stream defined as %d/%d/%d in order to change its state to %s!", thePort.getModule(), thePort.getPort(), state.isEnabled()));
	}
	
	@Override
	public streamTxStatistics getStreamTxStatistics(int logicalPortAddress, int streamID) throws Exception {
		//XenaStatistics stats = xena.PT_TOTAL(ports[logicalPortAddress].module, ports[logicalPortAddress].port);
		streamTxStatistics txStats = new streamTxStatistics(xena,xena.PT_STREAM_String(ports[logicalPortAddress].module, ports[logicalPortAddress].port ,streamID));
		
		txStats.numberOfFcsErrorsInjected = 0;
		
		for (XenaStream currentStream : streams) {
			if (currentStream.getSid() == streamID) {
				txStats.numberOfFcsErrorsInjected = currentStream.getNumberOfInjectedFcsErrors();
			}
		}
			
		return txStats;
	}
	
		
	@Override
	public streamRxStatistics getStreamRxStatistics(int streamID) throws Exception {
		long TxminusRx=0;
		XenaStatistics stats;
		int srcPort = streams.get(streamID - 1).getPort();
				
		int i=0;
		int srcModule = streams.get(streamID- 1).get_module();
		
		for (i = 0; i < ports.length; i++) {
			if (ports[i].getPort() == srcPort && ports[i].getModule() == srcModule)
				break;
		}
		
		streamTxStatistics txStats = getStreamTxStatistics(i, streamID);
		stats = getRxStatistics(streamID);
		streamRxStatistics rxStats = new streamRxStatistics(stats);
		TxminusRx= txStats.packets- rxStats.packets;
		int logicalPortAddress = rxStats.getReceivedPortNumber();
		rxStats.testPayloadErrors = xena.PR_TPLDERRORS(ports[logicalPortAddress].module, ports[logicalPortAddress].port, streamID);		
		rxStats.testPayloadErrors.setTxMinusRxPackets(TxminusRx);

		return rxStats;
	}
	
	class XenaPort 
	{
		int port;
		int module;
		int speed = 1000;
		protected boolean autoneg = true;
		protected boolean Enable = false;
		protected String portComment;
		public boolean getEnable() {
			return Enable;
		}
		public void setEnable(boolean enable) throws Exception{
			this.Enable = enable;
			xena.P_TRAFFIC(this.module, this.port, enable);
			
		}
		public XenaPort(int port, int module) {
			super();
			this.port = port;
			this.module = module;
		}
		public int getSpeed() throws Exception {
			speed = xena.P_SPEED(this.module, this.port);
			return speed;
		}
		public void setSpeed(int speed) throws Exception {
			this.speed = speed;
			xena.P_SPEED(this.module, this.port, speed);

		}
		
		public void setPortComment() throws Exception{
			xena.P_COMMENT(this.module,this.port, portComment);
		}
		public boolean isAutoneg() {
			return autoneg;
		}
 
		public void setAutoneg(boolean autoneg) throws Exception {
			xena.P_AUTONEG(this.module, this.port, autoneg);
			this.autoneg = autoneg;						
		}
		
		
		public int getPort() {
			return port;
		}
		public void setPort(int port) {
			this.port = port;
		}
		public int getModule() {
			return module;
		}
		public void setModule(int module) {
			this.module = module;
		}
	}

	protected boolean isDebug() {
		return debug;
	}


	protected void setDebug(boolean debug) {
		this.debug = debug;
	}
	@Override
	public boolean isPortSynced(int logicalPortAddress) throws Exception {
		return(xena.P_SYNC(ports[logicalPortAddress].module, ports[logicalPortAddress].port));
	}
	@Override
	public String portSpeedStatus(int logicalPortAddress) throws Exception{
		int Speed;
		Speed = xena.P_SPEED(ports[logicalPortAddress].module, ports[logicalPortAddress].port);
		if (Speed == 10) {
			return "10fd";
		}
		if (Speed == 100) {
			return "100fd";
		}
		if (Speed == 1000) {
			return "1000fd";
		}
		return Integer.toString(Speed);
		
	}
	
	
	
	@Override
	public String isPortSyncedStatus(int logicalPortAddress) throws Exception {
		return(xena.P_SYNC_STATUS(ports[logicalPortAddress].module, ports[logicalPortAddress].port));
	}
	
		
	public boolean preparePortForCapture(int logicalPortAddress) throws Exception {
		boolean operationSucceeded = true;

		xena.PC_TRIGGER(ports[logicalPortAddress].module,  ports[logicalPortAddress].port, portCaptureStartStopCriteria.Immediately, 0, portCaptureStartStopCriteria.StopWhenFull);
		int captureLength;
		if (captureBytesToKeep < 0) {
			captureLength = -1;
		}
		else {
			captureLength = (captureBytesToKeep / 8) * 8;
		}
		xena.PC_KEEP(ports[logicalPortAddress].module,  ports[logicalPortAddress].port, portCaptureKeepCriteria.All, 0, captureLength);

		return operationSucceeded;
	}
	
	public boolean saveCapturedTrafficToFile(int logicalPortAddress, String captureFileFullPath) throws Exception {
		boolean operationSucceeded = true;

		byte[] bytes;
		FileOutputStream fos = new FileOutputStream(captureFileFullPath);
		// First step involves writing the pcap header to the file.
		pcapFileHeader fileHeader = new pcapFileHeader();
		pcapPacketHeader packetHeader = new pcapPacketHeader();

		bytes = fileHeader.toByteArray();
		fos.write(bytes);

		long numberOfPackets = xena.PC_STATS_NUMBER_OF_PACKETS_IN_BUFFER(ports[logicalPortAddress].module,  ports[logicalPortAddress].port);

		if (numberOfPackets == 0) {
			operationSucceeded = false;
		}
		else {
			if (packetCaptureLength != -1) {
				if (numberOfPackets > packetCaptureLength) {
					numberOfPackets = packetCaptureLength;
				}
			}
				
			for (int packetIndex = 0; packetIndex < numberOfPackets; packetIndex++) {
				PC_EXTRA_RESPONSE packetExtraInfo = xena.PC_EXTRA(ports[logicalPortAddress].module,  ports[logicalPortAddress].port, packetIndex);
				packetHeader.ts_sec = (int) (packetExtraInfo.time / Math.pow(10, 9));
				packetHeader.ts_usec = (int) (packetExtraInfo.time / Math.pow(10, 3));
				packetHeader.incl_len = packetExtraInfo.length; // Do not Remove FCS bytes
				packetHeader.orig_len = packetExtraInfo.length;
				bytes = packetHeader.toByteArray();
				fos.write(bytes);
				bytes = xena.PC_PACKET(ports[logicalPortAddress].module,  ports[logicalPortAddress].port, packetIndex);
				fos.write(bytes, 0, packetHeader.incl_len);
			}
		}

		fos.close();

		return operationSucceeded;
	}
	
	@Override
	public boolean capturePortTrafficToFile(int logicalPortAddress, String captureFileFullPath) throws Exception {
		boolean operationSucceeded = true;

		if (!preparePortForCapture(logicalPortAddress))
			return false;

		xena.P_CAPTURE(ports[logicalPortAddress].module,  ports[logicalPortAddress].port, captureDuration_ms);
		
		if (!saveCapturedTrafficToFile(logicalPortAddress, captureFileFullPath))
			operationSucceeded = false;
		
		return operationSucceeded;
	}
	
	@Override
	public void start() throws Exception {
		start(false);
		
	}
	
	@Override
	public boolean playbackCaptureFile(int logicalPortAddress, String captureFileFullPath) throws Exception {
		// Playback of a capture file on a specific port consists of the following procedure:
		// 1. Parse the header - not necessary to read it
		// 2. Loop the remainder of the file:
		//    1. Read the packet header and parse the packet's length
		//    2. Read the number of bytes specified by the packet length
		//    3. Send the bytes to Xena using: [Module/Port] P_XMITONE hexdata
		
		File file = new File(captureFileFullPath);
		byte[] singlePacketOrHeader = null;
		int fileSize = (int) file.length(), fileOffset = 0, packetNumber = 0;

		InputStream input = null;
		try {
			input = new BufferedInputStream(new FileInputStream(file));
			// Read the pcap file header
			singlePacketOrHeader = new byte[pcapFileHeader.getSize()];
			input.read(singlePacketOrHeader, 0, pcapFileHeader.getSize());
			pcapFileHeader fileHeader = pcapFileHeader.setFromByteArray(singlePacketOrHeader);
			if (fileHeader.isValid()) {
				fileOffset += pcapFileHeader.getSize();
				while (fileOffset < fileSize) {
					packetNumber++;
					// Read the packet's header
					singlePacketOrHeader = new byte[pcapPacketHeader.getSize()];
					input.read(singlePacketOrHeader, 0, pcapPacketHeader.getSize());
					pcapPacketHeader packetHeader = pcapPacketHeader.setFromByteArray(singlePacketOrHeader, fileHeader.magic_number);
					if (packetHeader.incl_len <= 0) {
						throw new Exception(String.format("Erroneous packet size (%d bytes) according to the header!", packetHeader.incl_len));
					}
					fileOffset += pcapPacketHeader.getSize();
					// Now read the actual packet's contents
					singlePacketOrHeader = new byte[packetHeader.incl_len];
					input.read(singlePacketOrHeader, 0, packetHeader.incl_len);
					fileOffset += packetHeader.incl_len;
					// Now send the packet to the designated port
					xena.P_XMITONE(ports[logicalPortAddress].module, ports[logicalPortAddress].port, singlePacketOrHeader);					
				}
			}
		} catch (Exception e) {
			throw new Exception(String.format("Error playing back %s on [%d/%d]! Error at offset 0x%x (packet #%d), error description: %s", 
					captureFileFullPath, ports[logicalPortAddress].module, ports[logicalPortAddress].port,
					fileOffset, packetNumber, e.getMessage()));
		}
		input.close();
		return true;
	}
	@Override
	public void setCaptureLength(int numberOfPacketsToCapture) throws Exception {
		if (numberOfPacketsToCapture == 0) {
			packetCaptureLength = defaultPacketCaptureLength;
		}
		else
			packetCaptureLength = numberOfPacketsToCapture;
	}

	@Override
	public int getStreamCount()
	{
		return streams.size();
	}
}
