package xena;
import static java.util.concurrent.TimeUnit.SECONDS;

import java.io.*;
import java.net.*;
import java.util.concurrent.Executors;
import java.util.concurrent.ScheduledExecutorService;
import java.util.concurrent.ScheduledFuture;
import org.springframework.util.StopWatch;

public class XenaCom {

	 
	public enum portCaptureStartStopCriteria {
		Immediately("ON", 0),
		StopWhenFull("FULL", 0),
		FcsError("FCSERR", 1),
		SatisfyingFilterCondition("FILTER", 2);
		
		protected String stringValue;
		protected int codedInteger;
		
		portCaptureStartStopCriteria(String _stringValue, int _codedInteger) {
			this.stringValue = _stringValue;
			this.codedInteger = _codedInteger;		
		}
		
		public String getString() {
			return stringValue;
		}

		/**
		 * @return the codedInteger
		 */
		public int getCodedInteger() {
			return codedInteger;
		}
	}

	public enum portCaptureKeepCriteria {
		All("ALL", 0),
		FcsError("FCSERR", 1),
		NoTestPayload("NOTPLD", 2),
		WithTestPayload("TPLD", 3),
		SatisfyingFilterCondition("FILTER", 4);
		
		protected String stringValue;
		protected int 	 codedInteger;
		
		portCaptureKeepCriteria(String _stringValue, int _codedInteger) {
			this.stringValue = _stringValue;
			this.codedInteger = _codedInteger;		
		}
		
		public String getString() {
			return stringValue;
		}
		/**
		 * @return the codedInteger
		 */
		public int getCodedInteger() {
			return codedInteger;
		}	
	}
	
	public class PC_EXTRA_RESPONSE {
		public long time;
		public long latency;
		public long ifg;
		public int length;
	}
	
	private Socket socket = null;
	PrintWriter out;
	BufferedReader in;
	Xena owner;
	private final ScheduledExecutorService scheduler =
	 	       Executors.newScheduledThreadPool(1);
	private double maxTimeout_sec = 6;
	
	public void setOwner(Xena xena)
	{
		owner = xena;
	}
	
	public void command(int module,int port,int _sid,String cmd,String param,int param1,int param2) throws Exception
	{
		command(String.format("%d/%d %s [%d] %s %d %d",module,port,cmd,_sid,param,param1,param2));
	}
	public void command(int module,int port,int _sid,String cmd,String param) throws Exception
	{
		command(String.format("%d/%d %s [%d] \"%s\"",module,port,cmd,_sid,param));
	}
	public void command(int module,int port,int _sid,String cmd,int param1,int param2) throws Exception
	{
		command(String.format("%d/%d %s [%d] %d %d",module,port,cmd,_sid,param1,param2));
	}

	public void command(int module,int port,String cmd1,int _sid,String cmd2) throws Exception
	{
		command(String.format("%d/%d %s [%d] %s",module,port,cmd1,_sid,cmd2));
	}

	/*
	 * 
	 */
	public void command(int module,int port,String cmd1,int _sid,String cmd2,String cmd3) throws Exception
	{
		command(String.format("%d/%d %s [%d] %s %s",module,port,cmd1,_sid,cmd2,cmd3));
	}
	//Command for exceptions--------------------------------------------//
	
	public void command(int module, int port, String cmd1, int _sid) throws Exception{
		command(String.format("%d/%d %s [%d]", module, port, cmd1, _sid));
	}
	//-------------------------------------------------------------------------//
	public void command(int module,int port,int _sid,String cmd,double param) throws Exception
	{
		if (param == (int)param)
			command(String.format("%d/%d %s [%d] %d",module,port,cmd,_sid,(int)param));
		else
			command(String.format("%d/%d %s [%d] %.2f",module,port,cmd,_sid,param));
	}
	
	public void command(int module,int port,int _sid,String cmd) throws Exception
	{
		command(String.format("%d/%d %s [%d]",module,port,cmd,_sid));
	}
	
	public void command(int module,int port,String cmd,boolean onoff) throws Exception
	{
		command(String.format("%d/%d %s %s",module,port,cmd,onoff ? "ON":"OFF"));
	}
	public void command(int module,int port,String cmd,String param) throws Exception
	{
		command(String.format("%d/%d %s \"%s\"",module,port,cmd,param));
	}	
	public void command(int module,int port,String cmd) throws Exception
	{
		command(String.format("%d/%d %s",module,port,cmd));
	}
	
	public void command(String cmd,boolean onoff) throws Exception
	{
		command(cmd + " " + (onoff ? "ON":"OFF"));
	}

	public void command(String cmd,String str) throws Exception
	{
		command(cmd + " \"" +str + "\"");
	}/* command to log in. Good for C_LOGON, C_OWNER, */
	
//	public void command(int module, int port, String cmd, Boolean turnOn) throws Exception
//	{
//	 command(String.format("%d/%d %s %",module, port, cmd));
//	}

	private void commandWithException (String cmd) throws Exception
	{
		StopWatch watch = new StopWatch();
		if (owner.isDebug())
			System.out.println("Xena command: "+ cmd);
		String info;
		synchronized (this) {
			out.println(cmd);
			
			watch.start("Waiting for incoming string");
			double dStart = System.currentTimeMillis();
			do
			{
				Thread.sleep(owner.getCommunicationSleep_msec());
				info = in.readLine();
				double elapsedTime_sec = (System.currentTimeMillis() - dStart) / 1000;
				if (elapsedTime_sec > maxTimeout_sec) {
				// We've been waiting for a response for too long - something's wrong.
					throw new Exception(String.format("Xena command %s failed to respond within %.2f seconds!", cmd, maxTimeout_sec));
				}
			} while (info == null);
			//System.out.println("Xena response: "+ info);
			if (!info.contains("<OK>") && !info.contains("C_KEEPALIVE"))
			{
				throw new Exception(String.format("Xena Failure. command='%s' response='%s'",cmd,info));
			}
		}
	}
	
	public void command(String cmd) throws Exception
	{
		try
		{
			commandWithException(cmd);
		} catch (SocketException e) {
			owner.reconnect();
			commandWithException(cmd);
		}
	} /* command to communicate with Xena*/

	
	
	public String ask(int module,int port,int sid,String cmd) throws Exception
	{
		return ask(module,port,String.format("%s [%d]",cmd,sid));
	}
	
	public String ask(int module,int port,String cmd,String offcet) throws Exception
	{
	    return  ask(module,port,String.format("%d [%d] %s %s",cmd,offcet));
		
	}
	
	public String ask(int module,int port,String cmd) throws Exception
	{
		String full_command = String.format("%d/%d %s",module,port,cmd);
		return ask(full_command);
	}
	
//	public String ask(int module,int port,String cmd,String cmd1) throws Exception
//	{
//		String full_command = String.format("%d/%d %s %s",module,port,cmd,cmd1);
//		return ask(full_command);
//	}
	
	
	private String askWithException(String cmd) throws Exception
	{	
		synchronized (this) {
			String full_command = String.format("%s ?",cmd);
			//System.out.println("Xena ask: " +full_command);
			out.println(full_command);
			String info;
			do
			{
				Thread.sleep(owner.getCommunicationSleep_msec());
				info = in.readLine();
			} while (info == null);
			info = info.replaceAll("\\s+"," ");
			if (owner.isDebug())
				System.out.println("Xena response: " + info);
			
			// the return values contains the command and then comes the response
			if (!info.startsWith(cmd))
			{
				throw new Exception(String.format(
						"Xena Failure. command='%s' response='%s'", full_command, info));
			}
			return info.substring(cmd.length()).trim();
		}
	}

	public String ask(String cmd) throws Exception
	{
		try
		{
			return askWithException(cmd);
		} catch (SocketException e) {
			owner.reconnect();
			return askWithException(cmd);
		}
	}


	public boolean askbool(int module,int port,int sid,String cmd) throws Exception
	{	
		String ret = ask(module,port,sid,cmd);
		if (ret.contains("ON"))
			return true;
		return false;
	}
	public boolean askbool(int module,int port,String cmd) throws Exception
	{	
		String ret = ask(module,port,cmd);
		if (ret.contains("ON"))
			return true;
		return false;
	}	
	
	public boolean askbool(String cmd) throws Exception
	{	
		String ret = ask(cmd);
		if (ret.contains("ON"))
			return true;
		return false;
	}
	
	public int askInt(String cmd) throws Exception
	{	
		String ret = ask(cmd);
		return Integer.parseInt(ret);
	}
	
	public int askInt(int module,int port,String cmd) throws Exception
	{	
		String ret = ask(module,port,cmd);
		return Integer.parseInt(ret);

	}	
	
	/*************************************************************************************************************/
	/*************************************************************************************************************/

	public void C_CONNECT(String _ipaddr) throws Exception {
		socket = new Socket(_ipaddr, 22611); // IP of Xena +
		socket.setKeepAlive(true); 
		out = new PrintWriter(socket.getOutputStream(), true);
		in = new BufferedReader(new InputStreamReader(socket.getInputStream()));
	}

	public void C_DISCONNECT() {
		if (socket != null)
		{
			try {
				socket.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
			socket = null;
			in = null;
			out = null;
		}
		scheduler.shutdown();
	}

	public void C_LOGON( String _pwd) throws Exception 
	{
		command("C_LOGON",_pwd);
		
	 	        final Runnable beeper = new Runnable() {
	 	                @Override
						public void run() { 
	 	                	System.out.println("beep"); 
	 	                	try {
	 	                		C_KEEPALIVE();
							} catch (Exception e) {
								e.printStackTrace();
							}
	 	                	}
	 	        };
	 	        final ScheduledFuture<?> beeperHandle =
	 	            scheduler.scheduleAtFixedRate(beeper, 30, 20, SECONDS);
	 	        scheduler.schedule(new Runnable() {
	 	                @Override
						public void run() { /*beeperHandle.cancel(true);*/ }
	 	            }, 0, SECONDS);
	}
	
	public void C_LOGOFF() throws Exception 
	{
		command("C_LOGOFF");
	}

	public void P_AUTONEG(int _module, int _port, Boolean turnOn) throws Exception 
	{
		
		command( _module, _port, "P_AUTONEGSELECTION", turnOn );
		
	}

	public void C_OWNER( String _username) throws Exception 
	{
		command("C_OWNER",_username);
	}

	public String C_OWNER() throws Exception 
	{
		return ask("C_LOGON");
	}

	public String C_KEEPALIVE() throws Exception 
	{
		return ask("C_KEEPALIVE");
	}

	public String C_RESERVEDBY() throws Exception 
	{
		return ask("C_RESERVEDBY");
	}
	
	public String C_MODEL() throws Exception 
	{
		return ask("C_MODEL");
	}
	
	public String C_PORTCONTS() throws Exception 
	{
		return ask("C_PORTCONTS");
	}
	
	public String C_CAPABILITIES() throws Exception 
	{
		return ask("C_CAPABILITIES");
	}

	public String C_SERIALNO() throws Exception 
	{
		return ask("C_SERIALNO");
	}	

	public String C_VERSIONNO() throws Exception 
	{
		return ask("C_VERSIONNO");
	}	

	public String C_NAME() throws Exception 
	{
		return ask("C_NAME");
	}	

	public void C_NAME( String _chassisname) throws Exception 
	{
		command("C_NAME" ,_chassisname);
	}

	public String C_COMMENT() throws Exception 
	{
		return ask("C_COMMENT");
	}	

	public void C_COMMENT( String _comment) throws Exception 
	{
		command("C_COMMENT" ,_comment);
	}

	public String C_PASSWORD() throws Exception 
	{
		return ask("C_PASSWORD");
	}	
	
	public void C_PASSWORD( String _password) throws Exception 
	{
		command("C_COMMENT" ,_password);
	}

	public String C_IPADDRESS() throws Exception 
	{
		return ask("C_IPADDRESS");
	}	
	
	public void C_IPADDRESS( String _ipaddr, String _mask, String _gw) throws Exception 
	{
		command("C_IPADDRESS " +_ipaddr + " " + _mask + " " + _gw);
	}

	
	public String C_INDICES() throws Exception 
	{
		return ask("C_INDICES");
	}

	public void C_DOWN(String _shutdownrestart)  throws Exception 
	{
		command("C_DOWN -1480937026" ,_shutdownrestart);
	}

	public void C_RESERVE()  throws Exception 
	{
		command("C_RESERVATION RESERVE");
	}
	
	public void C_RELEASE()  throws Exception 
	{
		command("C_RESERVATION RELEASE");
	}
	public void C_RELINQUISH()  throws Exception 
	{
		command("C_RESERVATION RELINQUISH");
	}	

	/*************************************************************************************************************/

	public void PC_TRIGGER(int _module, int _port, portCaptureStartStopCriteria startCondition, int filter, portCaptureStartStopCriteria stopCondition)  throws Exception {
		command(_module, _port, String.format("PC_TRIGGER %d %d %d %d", startCondition.getCodedInteger(), filter, stopCondition.getCodedInteger(), filter));
	}
	
	public void PC_KEEP(int _module, int _port, portCaptureKeepCriteria keepCondition, int index, int howManyBytesToKeep)  throws Exception {
		command(_module, _port, String.format("PC_KEEP %d %d %d", keepCondition.getCodedInteger(), index, howManyBytesToKeep));
	}

	public long PC_STATS_NUMBER_OF_PACKETS_IN_BUFFER(int _module, int _port) throws Exception {
		String response = ask(_module, _port, "PC_STATS");
		String[] allWords = response.split("\\s+");
		
		return Long.parseLong(allWords[1]);
	}

	public void P_CAPTURE(int _module, int _port, boolean captureState) throws Exception {
		command(_module, _port, "P_CAPTURE", captureState);
	}
	
	public void P_CAPTURE(int _module, int _port, int time_ms) throws Exception {
		P_CAPTURE(_module, _port, true);
		Thread.sleep(time_ms);
		P_CAPTURE(_module, _port, false);		
	}

	public void P_XMITONE(int _module, int _port, byte[] packetContent) throws Exception {
		String contentAsString = javax.xml.bind.DatatypeConverter.printHexBinary(packetContent);//  new String(packetContent);
		
		command(_module, _port, "P_XMITONE 0x" + contentAsString);	
	}
	
	public byte[] PC_PACKET(int _module, int _port, int packetId) throws Exception {
		String response = PC_PACKET_STRING(_module, _port, packetId);
		// Remove leading 0x 
		response = response.replace("0x", "");
		byte[] bytes = new byte[response.length() / 2];
		for (int index = 0; index < response.length(); index += 2) {
			String singleByte = response.substring(index, index + 2);
			bytes[index / 2] = (byte)Short.parseShort(singleByte, 16);
		}
		
		return bytes;		
	}
	
	protected String PC_PACKET_STRING(int _module, int _port, int packetId) throws Exception {
		String response = ask(_module, _port, packetId, "PC_PACKET");
		return response;
	}
	
	public PC_EXTRA_RESPONSE PC_EXTRA(int _module, int _port, int packetId) throws Exception {
		PC_EXTRA_RESPONSE extraResponse = new PC_EXTRA_RESPONSE();
		int index = 0;
		String response = ask(_module, _port, packetId, "PC_EXTRA");
		String[] allWords = response.split("\\s+");

		if (allWords.length < 4) {
			throw new Exception(String.format("PC_EXTRA: Expected %d bytes response, received %d bytes. Response string: %s", 4, allWords.length, response));
		}
		extraResponse.time = Long.parseLong(allWords[index++]);
		extraResponse.latency = Long.parseLong(allWords[index++]);
		extraResponse.ifg = Long.parseLong(allWords[index++]);
		extraResponse.length = Integer.parseInt(allWords[index++]);

		return extraResponse;
	}
	
	public void P_RESERVE( int _module, int _port)  throws Exception
	{
		command(_module,_port,"P_RESERVATION RESERVE");
	}
	
	public void P_RESERVE_SAFE( int _module, int _port,boolean force)  throws Exception
	{
		if (P_IS_RESERVE_BY_ME(_module,_port))
			return; 
		if (!P_IS_RESERVE_FREE(_module,_port))
		{
			if (force)
				P_RELINQUISH(_module,_port);
			else
			{
				String by = P_RSERVEDBY(_module,_port);
				throw new Exception("Xena port is in use by " + by);
			}
		}
		P_RESERVE(_module,_port);
	}
	public boolean P_IS_RESERVE_BY_ME( int _module, int _port)  throws Exception
	{
		String res = ask(_module,_port,"P_RESERVATION");
		return res.compareToIgnoreCase("RESERVED_BY_YOU")==0;
	}
	
	public boolean P_IS_RESERVE_FREE( int _module, int _port)  throws Exception
	{
		String res = ask(_module,_port,"P_RESERVATION");
		return res.compareToIgnoreCase("RELEASED")==0;
	}
	
	public void P_RELEASE( int _module, int _port)  throws Exception
	{
		command(_module,_port,"P_RESERVATION RELEASE");
	}
	
	public void P_RELINQUISH( int _module, int _port)  throws Exception
	{
		command(_module,_port,"P_RESERVATION RELINQUISH");
	}
	
	public String P_RSERVEDBY( int _module, int _port) throws Exception {
		return ask(_module,_port,"P_RSERVEDBY");
	}

	public void P_RESET( int _module, int _port)  throws Exception
	{
		command(_module,_port,"P_RESET");
	}

	public String P_COMMENT( int _module, int _port)  throws Exception
	{
		return ask (_module,_port,"P_COMMENT");
	}

	public void P_COMMENT( int _module, int _port,String _comment)  throws Exception
	{
		command(_module,_port,"P_COMMENT",_comment);
	}


	public boolean P_TRAFFIC( int _module, int _port)  throws Exception
	{
		return askbool(_module,_port,"P_TRAFFIC");
	}

	public void P_TRAFFIC( int _module, int _port,boolean onoff) throws Exception
	{
		command(_module,_port,"P_TRAFFIC",onoff);
	}
	
	public int P_SPEED( int _module, int _port) throws Exception
	{
		return askInt(_module,_port,"P_SPEED");
	}
	

	public void P_SPEED( int _module, int _port,int speed) throws Exception
	{
		String param = "AUTO";
		switch (speed) {
		case 10: 
			param = "F10M";
			break;
		case 100: 
			param = "F100M";
			break;
		case 1000: 
			param = "F1G";
			break;
		default:
			break;
		}
		command(_module,_port,"P_SPEEDSELECTION " + param);
	}
	/*
	 * _module  -(Integer) The module number on xena
	 * _port   - (Integer)The port number   on xena
	 * _sid    - (Integer)The sub-index value of the stream definition
	 * @ by Gideon Gur Arie
	 */
	public void PS_PAYLOAD(int _module, int _port, int _sid, String payloadType, String payloadPattern) throws Exception {
		command(_module,_port,"PS_PAYLOAD",_sid, payloadType + " " + payloadPattern);		
	}
	
	public void PS_PAYLOAD(int _module, int _port ,int _sid)throws Exception{		
		command(_module,_port,"PS_PAYLOAD",_sid,"PRBS");
	}
	
	public boolean P_SYNC( int _module, int _port) throws Exception
	{
		String ret =  ask(_module,_port,"P_RECEIVESYNC");
		return ret.contains("IN_SYNC") ? true:false;
			
	}
	
	public String P_SYNC_STATUS( int _module, int _port) throws Exception
	{
		String ret =  ask(_module,_port,"P_RECEIVESYNC");
		return ret.contains("IN_SYNC") ? "up":"down";
			
	}

	/*************************************************************************************************************/
	public void PS_CREATE(  int _module, int _port, int _sid) throws Exception
	{
		command(_module,_port,_sid,"PS_CREATE");
	}

	public void PS_DELETE(  int _module, int _port, int _sid) throws Exception
	{
		command(_module,_port,_sid,"PS_DELETE");
	}

	public void PS_ENABLE(  int _module, int _port, int _sid,boolean enable) throws Exception
	{
		command(String.format("%d/%d PS_ENABLE [%d] %s",_module,_port,_sid,enable ? "ON":"SUPPRESS"));
	}

	public boolean PS_ENABLE(  int _module, int _port, int _sid) throws Exception
	{
		return askbool(_module,_port,_sid,"PS_ENABLE");
	}

	public String PS_PACKETLIMIT( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_PACKETLIMIT");
	}

	
	public void PS_PACKETLIMIT( int _module, int _port,int _sid, int _count) throws Exception
	{
		command(_module,_port,_sid,"PS_PACKETLIMIT",_count);
	}

	public String PS_COMMENT( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_COMMENT");
	}


	public void PS_COMMENT( int _module, int _port,int _sid, String _comment)  throws Exception
	{
		 command(_module,_port,_sid,"PS_COMMENT",_comment);
	}

	public String PS_TPLDID( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_TPLDID");
	}


	public void PS_TPLDID( int _module, int _port,int _sid, int _tpldid)  throws Exception
	{
		 command(_module,_port,_sid,"PS_TPLDID",_tpldid);
	}

	public String PS_RATEFRACTION( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_RATEFRACTION");
	}


	public void PS_RATEFRACTION( int _module, int _port,int _sid, double _fraction)  throws Exception
	{
		 command(_module,_port,_sid,"PS_RATEFRACTION",_fraction);
	}

	public String PS_RATEPPS( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_RATEPPS");
	}


	public void PS_RATEPPS( int _module, int _port,int _sid, int _pps)  throws Exception
	{
		 command(_module,_port,_sid,"PS_RATEPPS",_pps);
	}

	public String PS_RATEL2BPS( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_RATEL2BPS");
	}


	public void PS_RATEL2BPS( int _module, int _port,int _sid, double _bps)  throws Exception
	{
		 command(_module,_port,_sid,"PS_RATEL2BPS",(int)_bps);
	}

	public String PS_RATE( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_RATE");
	}
	public String PS_BURST( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_BURST");
	}
	
	public void PS_BURST( int _module, int _port,int _sid, int _size,int _density)  throws Exception
	{
		 command(_module,_port,_sid,"PS_BURST",_size,_density);
	}
	
	public String PS_PACKETHEADER( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_PACKETHEADER");
	}

	public void PS_PACKETHEADER( int _module, int _port,int _sid,String data)  throws Exception
	{
		 command(String.format("%d/%d PS_PACKETHEADER [%d] %s",_module,_port,_sid,data));
		//command(_module, _port, _sid,"PS_PACKETHEADER", data);
	}

	
	public String PS_HEADERPROTOCOL( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_HEADERPROTOCOL");
	}

	public void PS_HEADERPROTOCOL( int _module, int _port,int _sid,String data)  throws Exception
	{
		 command(String.format("%d/%d PS_HEADERPROTOCOL [%d] %s",_module,_port,_sid,data));
		//command(_module,_port, _sid, "PS_HEADERPROTOCOL", data);
	}	
	
	public String PS_MODIFIERCOUNT( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_MODIFIERCOUNT");
	}


	public void PS_MODIFIERCOUNT( int _module, int _port,int _sid, int _count)  throws Exception
	{
		 command(_module,_port,_sid,"PS_MODIFIERCOUNT",_count);
	}

	public void PS_MODIFIER( int _module, int _port,int _mid, int _sid, int _pos, int _mask, String _act, int _rep) throws Exception {
		command(_module, _port, String.format("PS_MODIFIER [%d,%d] %d 0x%08x %s %d", _sid, _mid, _pos, _mask, _act, _rep));		
	}
	
	public void PS_MODIFIERRANGE( int _module, int _port,int _mid, int _sid, int _start, int _step, int _stop) throws Exception {
		command(_module, _port, String.format("PS_MODIFIERRANGE [%d,%d] %d %d %d", _sid, _mid, _start, _step, _stop));		
	}

	
	public void PS_PACKETLENGTH( int _module, int _port,int _sid,String type, int _min, int _max)  throws Exception
	{
		 command(_module,_port,_sid,"PS_PACKETLENGTH",type,_min,_max);
	}

	public String PS_PACKETLENGTH( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PS_PACKETLENGTH");
	}
	
	public String PS_INDICES( int _module, int _port) throws Exception
	{
		return ask(_module,_port,"PS_INDICES");
	}	

	public int PS_COUNT( int _module, int _port) throws Exception
	{
		String str = PS_INDICES(_module,_port).trim();
		if (str.isEmpty())
			return 0;
		String sp[] = str.split(" ");
		return sp.length;
	}	
	public String PT_STREAM_String( int _module, int _port,int _sid) throws Exception
	{
		return ask(_module,_port,_sid,"PT_STREAM");
	}
	public void PS_INJECTFCSERR(int _module, int _port, int _sid) throws Exception{
		command(_module, _port, "PS_INJECTFCSERR", _sid);
	}
	public void PS_INJECTTPLDERR(int _module, int _port, int _sid) throws Exception{
		command(_module, _port, "PS_INJECTTPLDERR", _sid);
	}
	
	//----------------------------------------------------------------------//
	public XenaStatistics PT_TOTAL( int _module, int _port) throws Exception
	{
		return new XenaStatistics(ask(_module,_port,"PT_TOTAL"));
	}
	
	public XenaStatistics PR_TOTAL( int _module, int _port) throws Exception
	{
		return new XenaStatistics(ask(_module,_port,"PR_TOTAL"));
	}

	public XenaStatistics PT_STREAM( int _module, int _port,int _sid) throws Exception
	{
		return new XenaStatistics(ask(_module,_port,_sid,"PT_STREAM"));
	}
	
	public XenaStatistics PR_STREAM( int _module, int _port,int _sid) throws Exception
	{
		return new XenaStatistics(ask(_module,_port,_sid,"PR_TPLDTRAFFIC"));
	}
	
	
	/*
	 * 
	 */
/*	public  XenaStatistics PR_TPLDERRORS(int _module, int _port,int _sid)throws Exception
	{
		return new  XenaStatistics(ask(_module,_port,_sid,"PR_TPLDERRORS"));
	}
	*/
	public payloadErrors PR_TPLDERRORS(int _module, int _port,int _sid)throws Exception
	{
		return new  payloadErrors(ask(_module,_port,_sid,"PR_TPLDERRORS"));
	} 
	
	public XenaLatencyJitterStatistics PR_TPLDJITTER( int _module, int _port,int _sid) throws Exception {
		return new XenaLatencyJitterStatistics(ask(_module,_port,_sid,"PR_TPLDJITTER"));
	}

	public XenaLatencyStatistics PR_STREAM_LATENCY( int _module, int _port,int _sid) throws Exception
	{
		return new XenaLatencyStatistics(ask(_module,_port,_sid,"PR_TPLDLATENCY"));
	}
	
	public XenaLatencyStatistics PR_STREAM_LATENCY( int _module, int _port) throws Exception
	{
		return new XenaLatencyStatistics(ask(_module,_port,"PR_TPLDLATENCY", "1"));
	}
	
	
	
	
	public void PT_CLEAR( int _module, int _port)  throws Exception
	{
		 command(_module,_port,"PT_CLEAR");
	}	
	public void PR_CLEAR( int _module, int _port)  throws Exception
	{
		 command(_module,_port,"PR_CLEAR");
	}

	public void P_CLEAR( int _module, int _port)  throws Exception
	{
		PT_CLEAR(_module,_port);
		PR_CLEAR(_module,_port);
	}
	
	public class XenaStatistics
	{
		public final long bps;
		public final long pps;
		public final long bytes;
		public final long packets;
		public final long prbs;
		protected int receivedPortNumber;
		public int getReceivedPortNumber() {
			return receivedPortNumber;
		}

		public void setReceivedPortNumber(int receivedPortNumber) {
			this.receivedPortNumber = receivedPortNumber;
		}

		XenaStatistics (String data)
		{
			String st[] = data.split(" ");
			bps = Long.parseLong(st[0]);
			pps = Long.parseLong(st[1]);
			bytes = Long.parseLong(st[2]);
			packets = Long.parseLong(st[3]);
			prbs = Long.parseLong(st[3]);
		}

		@Override
		public String toString() {
			return String.format("Mbps = %.2f pps = %d bytes = %d packets = %d",(double)bps / 1000000, pps, bytes, packets);
		}
		
		
	}

	public class XenaLatencyJitterStatistics
	{
		public XenaLatencyJitterStatistics(String data) {
			String st[] = data.split(" ");
			String s= st[0];
			System.out.println(s);
			min = Long.parseLong(st[0]);
			avg = Long.parseLong(st[1]);
			max = Long.parseLong(st[2]);
		}

		public long min;
		public long avg;
		public long max;
	}
	
	public class XenaLatencyStatistics
	{
		public long min;
		public long max;
		public long avg;
		public XenaLatencyJitterStatistics jitter;

		XenaLatencyStatistics (String data)
		{
			String st[] = data.split(" ");
			String s= st[0];
			System.out.println(s);
			min = Long.parseLong(st[0]);
			avg = Long.parseLong(st[1]);
			max = Long.parseLong(st[2]);
			
		}

		@Override
		public String toString() {
			return String.format("min = %d avg = %d max = %d",min,avg,max);
		}
		
		public XenaLatencyStatistics merge(XenaLatencyStatistics stat)
		{
			return new XenaLatencyStatistics(Math.min(min, stat.min),Math.max(max,stat.max),(avg+stat.avg)/2);

		}

		public XenaLatencyStatistics(long min, long max, long avg) {
			super();
			this.min = min;
			this.max = max;
			this.avg = avg;
		}
				
	}
	
}
