package xena;

import java.util.ArrayList;



public class XenaPacket {
	// ======================================= ENUMERATIONS ====================================
	public enum packetSizeTypes {
		Fixed("FIXED"),
		Random("RANDOM"),
		Increment("INCREMENTING");
		
		String patternString;
		private packetSizeTypes(String _patternString) {
			patternString = _patternString;
		}

		/**
		 * @return the patternString
		 */
		public String getString() {
			return patternString;
		}
		
		
	}
	
	public enum packetHeaderTypes {
		ETH("Ethernet"),
		CVLAN("CVLAN"),
		SVLAN("SVLAN"),
		IP("IP"),
		MPLS("MPLS");
		
		String theString;
		private packetHeaderTypes(String _theString) {
			theString = _theString;
		}
		public String getString() {
			return theString;
		}
		
		
	}
	public enum modifierFieldTypes {
		DestinationMAC,
		SourceMAC,
		EthernetType,
		vlanPriority,
		vlanId,		
		cfi,
		DifServCP,
		Exp
	}
	
	public enum modifierAction {
		Increment("INC"),
		Decrement("DEC"),
		Random("RANDOM");
		
		String theString;
		private modifierAction(String _theString) {
			theString = _theString;
		}
		public String getString() {
			return theString;
		}		
	}
	
	public enum packetPayloadType {
		PRBS("PRBS"),
		Increment("INCREMENTING"),
		Pattern("PATTERN");
		
		String theString;
		private packetPayloadType(String _theString) {
			theString = _theString;
		}
		public String getString() {
			return theString;
		}
		
		public static packetPayloadType toEnum(String _str) throws Exception {
			packetPayloadType type = PRBS;
			packetPayloadType[] allTypes = type.getDeclaringClass().getEnumConstants();
			
			for (packetPayloadType singleType: allTypes) {
				if (_str.compareToIgnoreCase(singleType.getString()) == 0)
					return singleType;
			}
			throw new Exception(String.format("Unsupported packet payload type %s!", _str));
		}
	}
	// =========================================================================================
	
	packetSizeTypes packetSize;
	int packetSizeMin;
	int packetSizeMax;
	int sourceMacAddr;
	int destMacAddr;
	int stopAfterNumOfPackets;
	int headerLength;
	String packetName;
	ArrayList<packetHeader> header;
	packetPayloadType payloadType; 
	int payloadPattern;		


	public XenaPacket(packetSizeTypes _packetSize, int _packetSizeMin, int _packetSizeMax, int _stopAfterNumOfPackets) {
		packetSize = _packetSize;
		packetSizeMin = _packetSizeMin;
		packetSizeMax = _packetSizeMax;
		stopAfterNumOfPackets = _stopAfterNumOfPackets;
		header = new ArrayList<packetHeader>();
	}
	
	public XenaPacket(packetSizeTypes _packetSize, int _packetSizeFixed, int _stopAfterNumOfPackets) {
		packetSize = _packetSize;
		packetSizeMin = _packetSizeFixed;
		packetSizeMax = _packetSizeFixed;
		stopAfterNumOfPackets = _stopAfterNumOfPackets;
		header = new ArrayList<packetHeader>();
	}

	public short vid;
	public byte pcp;
	public byte cfi;
	public byte dscp;
	public byte Exp;
///--------------------------EthernetHeader-----------------------
	public boolean addEthernetHeader(long srcMac, long destMac) {
		return innerAddEthernetHeader(srcMac, destMac, null);
	}

	public boolean addEthernetHeader(long srcMac, long destMac, headerModifier modifier) {
		ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
		modifiers.add(modifier);
		return innerAddEthernetHeader(srcMac, destMac, modifiers);
	}

	public boolean addEthernetHeader(long srcMac, long destMac, ArrayList<headerModifier> modifiers) {
		return innerAddEthernetHeader(srcMac, destMac, modifiers);
	}

	protected boolean innerAddEthernetHeader(long srcMac, long destMac, ArrayList<headerModifier> modifiers) {
		packetHeader newHeader = new packetHeader(packetHeaderTypes.ETH);
		newHeader.srcMac = srcMac;
		newHeader.destMac = destMac;
		if (modifiers != null) {
			newHeader.modifiers = modifiers;
		}
		header.add(newHeader);
		return true;	
	}
	//--------------------------CVlanHeader-----------------------	
	public boolean addCVlanHeader(int vid, int priority, int cfi)
	 {
		innerAddCVlanHeader(vid, priority, cfi,0x8100,  null);
		return true;
	}
	public boolean addCVlanHeader(int vid, int priority, int cfi, headerModifier modifier) {
		ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
		modifiers.add(modifier);
		return innerAddCVlanHeader(vid, priority, cfi,0x8100, modifiers);
	}

	public boolean addCVlanHeader(int vid, int priority, int cfi, int eth_type)
	 {
		innerAddCVlanHeader(vid, priority, cfi,eth_type,  null);
		return true;
	}
	public boolean addCVlanHeader(int vid, int priority, int cfi,int eth_type, headerModifier modifier) {
		ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
		modifiers.add(modifier);
		return innerAddCVlanHeader(vid, priority, cfi,eth_type, modifiers);
	}
	protected boolean innerAddCVlanHeader(int vid, int priority, int cfi,int eth_type, ArrayList<headerModifier> modifiers) {
		packetHeader newHeader = new packetHeader(packetHeaderTypes.CVLAN);
		newHeader.priority = priority;
		newHeader.cfi = cfi;
		newHeader.vid=vid;
		newHeader.type= eth_type;
		if (modifiers != null) {
			newHeader.modifiers = modifiers;
		}
		header.add(newHeader);
		return true;	
	}
	//--------------------------SVlanHeader-----------------------	
	public boolean addSVlanHeader(int vid, int priority, int cfi) {
		return innerAddSVlanHeader(vid, priority, cfi,0x88A8, null);
	}
	
	
	public boolean addSVlanHeader(int vid, int priority, int cfi, headerModifier modifier) {
		ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
		modifiers.add(modifier);
		return innerAddSVlanHeader(vid, priority, cfi,0x88A8, modifiers);
	}
	
	public boolean addSVlanHeader(int vid, int priority, int cfi, int eth_type) {
		return innerAddSVlanHeader(vid, priority, cfi,eth_type, null);
	}
	
	
	public boolean addSVlanHeader(int vid, int priority, int cfi,int eth_type, headerModifier modifier) {
		ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
		modifiers.add(modifier);
		return innerAddSVlanHeader(vid, priority, cfi,eth_type, modifiers);
	}
	protected boolean innerAddSVlanHeader(int vid, int priority, int cfi,int eth_type, ArrayList<headerModifier> modifiers) {
		packetHeader newHeader = new packetHeader(packetHeaderTypes.SVLAN);
		newHeader.priority = priority;
		newHeader.cfi = cfi;
		newHeader.vid=vid;
		newHeader.type=eth_type;
		if (modifiers != null) {
			newHeader.modifiers = modifiers;
		}
		header.add(newHeader);
		return true;	
	}
	//--------------------------MPLSHeader-----------------------	
		public boolean addMPLSHeader(int Label,int Exp,int Bottom, int TimeToLive) {
			return innerAddMPLSHeader(Label, Exp, Bottom, TimeToLive,0x8847, null);
		}
		
		public boolean addMPLSHeader(int Label,int Exp,int Bottom, int TimeToLive, headerModifier modifier) {
			ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
			modifiers.add(modifier);
			return innerAddMPLSHeader(Label, Exp, Bottom, TimeToLive,0x8847, modifiers);
		}
		
		public boolean addMPLSHeader(int Exp, headerModifier modifier) {
			ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
			modifiers.add(modifier);
			return innerAddMPLSHeader( Exp,0x8847, modifiers);
		}
		public boolean addMPLSHeader(int Exp) {
			return innerAddMPLSHeader(Exp,0x8847, null);
		}
		
		public boolean addMPLSHeader(int Label,int Exp,int Bottom, int TimeToLive, int eth_type) {
			return innerAddMPLSHeader(Label, Exp, Bottom, TimeToLive,eth_type, null);
		}
		
		public boolean addMPLSHeader(int Label,int Exp,int Bottom, int TimeToLive,int eth_type,  headerModifier modifier) {
			ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
			modifiers.add(modifier);
			return innerAddMPLSHeader(Label, Exp, Bottom, TimeToLive,eth_type, modifiers);
		}
		protected boolean innerAddMPLSHeader(int Label,int Exp,int Bottom, int TimeToLive,int eth_type, ArrayList<headerModifier> modifiers) {
			packetHeader newHeader = new packetHeader(packetHeaderTypes.MPLS);
			newHeader.Label = Label;
			newHeader.Exp = Exp;
			newHeader.Bottom=Bottom;
			newHeader.TimeToLive=TimeToLive;
			newHeader.type=eth_type;
			if (modifiers != null) {
				newHeader.modifiers = modifiers;
			}
			header.add(newHeader);
			return true;	
		}
		public boolean addMPLSHeader(int Exp,int eth_type,  headerModifier modifier) {
			ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
			modifiers.add(modifier);
			return innerAddMPLSHeader( Exp,eth_type, modifiers);
		}
		public boolean addMPLSHeader(int Exp, int eth_type) {
			return innerAddMPLSHeader(Exp,eth_type,  null);
		}
		protected boolean innerAddMPLSHeader(int Exp,int eth_type,   ArrayList<headerModifier> modifiers) {
			packetHeader newHeader = new packetHeader(packetHeaderTypes.MPLS);
			newHeader.Label = 10;
			newHeader.Exp = Exp;
			newHeader.Bottom=1;
			newHeader.TimeToLive=250;
			newHeader.type=eth_type;
			if (modifiers != null) {
				newHeader.modifiers = modifiers;
			}
			header.add(newHeader);
			return true;	
		}
	//--------------------------IPHeader-----------------------	
	public boolean addIPHeader(int DifServCP){

		this.innerAddIPHeader(DifServCP,0x800, null);
		packetHeader newHeader = new packetHeader(packetHeaderTypes.IP);
//		newHeader.Version = 0xF000;
//		newHeader.DifServCP=DifServCP;
//		newHeader.ECN=1;
//		newHeader.TLEngth=20;
//		newHeader.Id=0x0000;
//		newHeader.Flags=2;
//		newHeader.FOffset=0;
//		newHeader.TLive=0x1080;
//		newHeader.Protocol=0x0663B3;
//		newHeader.HChecksum=0x63B3;
//		newHeader.Src_IP=0;
//		newHeader.Dst_IP=0;	
		return true;
	}

	public boolean addIPHeader(int Version,int DifServCP,int ECN,int TLEngth, int Id,int Flags, int FOffset, int TLive, int Protocol,
			int HChecksum, int Src_IP, int Dst_IP, ArrayList<headerModifier> modifiers) {
		return (innerAddIPHeader(Version,DifServCP,ECN,TLEngth, Id,Flags, FOffset, TLive, Protocol, HChecksum, Src_IP, Dst_IP,0x800, modifiers));
	}
	public boolean addIPHeader(int Version,int DifServCP,int ECN,int TLEngth, int Id,int Flags, int FOffset, int TLive, int Protocol,
				int HChecksum, int Src_IP, int Dst_IP) {
			addIPHeader(Version,DifServCP,ECN,TLEngth, Id,Flags, FOffset, TLive, Protocol, HChecksum, Src_IP, Dst_IP, null);
			return true;
		}
		
		public boolean addIPHeader(int DifServCP, headerModifier modifier) {
			ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
			modifiers.add(modifier);
			return innerAddIPHeader(DifServCP,0x800, modifiers);
		}			
		
		//
		public boolean addIPHeader(int DifServCP, int eth_type){

			this.innerAddIPHeader(DifServCP,eth_type, null);
			packetHeader newHeader = new packetHeader(packetHeaderTypes.IP);
//			newHeader.Version = 0xF000;
//			newHeader.DifServCP=DifServCP;
//			newHeader.ECN=1;
//			newHeader.TLEngth=20;
//			newHeader.Id=0x0000;
//			newHeader.Flags=2;
//			newHeader.FOffset=0;
//			newHeader.TLive=0x1080;
//			newHeader.Protocol=0x0663B3;
//			newHeader.HChecksum=0x63B3;
//			newHeader.Src_IP=0;
//			newHeader.Dst_IP=0;	
			return true;
		}

		public boolean addIPHeader(int Version,int DifServCP,int ECN,int TLEngth, int Id,int Flags, int FOffset, int TLive, int Protocol,
				int HChecksum, int Src_IP, int Dst_IP,int eth_type, ArrayList<headerModifier> modifiers) {
			return (innerAddIPHeader(Version,DifServCP,ECN,TLEngth, Id,Flags, FOffset, TLive, Protocol, HChecksum, Src_IP, Dst_IP,eth_type, modifiers));
		}
		public boolean addIPHeader(int Version,int DifServCP,int ECN,int TLEngth, int Id,int Flags, int FOffset, int TLive, int Protocol,
					int HChecksum, int Src_IP, int Dst_IP, int eth_type) {
				addIPHeader(Version,DifServCP,ECN,TLEngth, Id,Flags, FOffset, TLive, Protocol, HChecksum, Src_IP, Dst_IP,eth_type, null);
				return true;
			}
			
			public boolean addIPHeader(int DifServCP,int eth_type, headerModifier modifier) {
				ArrayList<headerModifier> modifiers = new ArrayList<headerModifier>();
				modifiers.add(modifier);
				return innerAddIPHeader(DifServCP,eth_type, modifiers);
			}
			
			protected boolean innerAddIPHeader(int DifServCP,int eth_type, ArrayList<headerModifier> modifiers){
				packetHeader newHeader = new packetHeader(packetHeaderTypes.IP);	
				this.innerAddIPHeader(5, DifServCP, 0, 20, 0, 0, 0, 128, 255, 0xC656, 15,16,eth_type, modifiers);
//				newHeader.Version = 0xF000;
//				newHeader.DifServCP=DifServCP;
//				newHeader.ECN=1;
//				newHeader.TLEngth=20;
//				newHeader.Id=0x0000;
//				newHeader.Flags=2;
//				newHeader.FOffset=0;
//				newHeader.TLive=0x1080;
//				newHeader.Protocol=0x0663B3;
//				newHeader.HChecksum=0x63B3;
//				newHeader.Src_IP=0;
//				newHeader.Dst_IP=0;	
//				if (modifiers != null) {
//					newHeader.modifiers = modifiers;
//				}
				//header.add(newHeader);
				return true;
			}
			protected boolean innerAddIPHeader(int Version,int DifServCP,int ECN,int TLEngth, int Id,int Flags, int FOffset, int TLive, int Protocol,
					int HChecksum, int Src_IP, int Dst_IP,int eth_type, ArrayList<headerModifier> modifiers) {
				packetHeader newHeader = new packetHeader(packetHeaderTypes.IP);
				newHeader.Version=Version;
				newHeader.DifServCP=DifServCP;
				newHeader.ECN =ECN;	
				newHeader.TLEngth =TLEngth;
				newHeader.Id =Id;
				newHeader.Flags =Flags;
				newHeader.FOffset =FOffset;
				newHeader.TLive =TLive;
				newHeader.Protocol =Protocol;
				newHeader.HChecksum =HChecksum;
				newHeader.Src_IP =Src_IP;
				newHeader.Dst_IP =Dst_IP;
				newHeader.type= eth_type;
				if (modifiers != null) {
					newHeader.modifiers = modifiers;
				}
				header.add(newHeader);
				return true;	
			}
		//--------------------------IPHeader-----------------------	
		
		
//	protected boolean addCVlanHeader(byte vid, byte pcp, short type, int priority, byte cfi)
//	 {
//		packetHeader newHeader = new packetHeader(packetHeaderTypes.CVLAN);
//		newHeader.vid =vid;
//		newHeader.pcp = pcp;
//		newHeader.type = type;
//		newHeader.priority=priority;
//		newHeader.cfi=cfi;
//		header.add(newHeader);
//		return true;
//	}
//	protected boolean addSVlanHeader(byte vid, byte pcp, short type, int priority, byte cfi) {
//		packetHeader newHeader = new packetHeader(packetHeaderTypes.SVLAN);
//		newHeader.vid =vid;
//		newHeader.pcp = pcp;
//		newHeader.type = type;
//		newHeader.priority=priority;
//		newHeader.cfi=cfi;
//		header.add(newHeader);
//		return true;
//	}
	public XenaPacket() {
		this(packetSizeTypes.Fixed, 64, 64, 0);		
	}
	
	
	// =========================================================================================
}
