package xena;

import java.util.ArrayList;

import jsystem.framework.report.Reporter;
import ethernetPacket.EthernetLayer;
import ethernetPacket.EthernetPacket;
import xena.XenaAPI.streamRateTypes;
import xena.XenaAPI.streamStates;
import xena.XenaCom.XenaLatencyStatistics;
import xena.XenaCom.XenaStatistics;
import xena.XenaPacket.packetPayloadType;
import xena.XenaPacket.packetSizeTypes;

public class XenaStream {
	private static int nextStreamIndex = 1;
	private int _sid;
	private int _module;
	private int _port;
	private int logicalPort;

	private XenaCom xena;
	private Xena manager;
	private EthernetPacket packet;
    private EthernetLayer  macaddress;
    private streamRate rate;
    private streamBurstParams burst;
    private streamStates state;
	private packetPayloadType payloadType;
	private String payloadPattern;
	private int numberOfInjectedFcsErrors = 0;
	private int numberOfInjectedTPLDErrors = 0;
	private int _maxNumOfModifiers = 2;
    public boolean setPayloadTypeAndPattern(packetPayloadType payloadType, String payloadPattern) throws Exception {
    	this.payloadType = payloadType;
    	this.payloadPattern = payloadPattern;
    	
    	xena.PS_PAYLOAD(_module, _port, _sid, payloadType.getString(), payloadPattern);
    	
    	return true;
    }

	public XenaStream(Xena manager,int logicalPort, int module, int port, String name,
			EthernetPacket packet, packetSizeTypes packetSize, int packetSizeMin, int packetSizeMax, streamBurstParams _burst, int _stopAfterNumberOfPackets) throws Exception {
		_sid = nextStreamIndex++;
		_module = module;
		_port = port;
		this.logicalPort = logicalPort;
		xena = manager.xena;
		this.manager = manager;
		this.packet = packet;
		packet.setName(name);
		xena.PS_CREATE(_module, _port, _sid);
		xena.PS_COMMENT(_module, _port, _sid, name);
		xena.PS_TPLDID(_module, _port, _sid, _sid);
		//xena.PS_PAYLOAD(_module, _port, _sid);
		macaddress = new EthernetLayer();
		this.burst = _burst;
		
		// Set the packet size
		switch (packetSize) {
		case Fixed:
		case Random:
		case Increment:
			xena.PS_PACKETLENGTH(_module, _port, _sid, packetSize.patternString, packetSizeMin, packetSizeMax);
			break;
		default:
			throw new Exception(String.format("Unsupported packet size type %d!", packetSize));
		}
		if (_stopAfterNumberOfPackets > 0)
			xena.PS_PACKETLIMIT(_module, _port, _sid, _stopAfterNumberOfPackets);
		else
			xena.PS_PACKETLIMIT(_module, _port, _sid, -1);

		enable(true);

	}
	
	public int getLogicalPort()
	{
		return logicalPort;
	}

	public void setLogicalPort(int logicalPort)
	{
		this.logicalPort = logicalPort;
	}
	public int getSid() {
		return _sid;
	}

	public void setSid(int _sid) {
		this._sid = _sid;
	}

	public static void ResetStreamNumber() 
	{
		nextStreamIndex = 1;
	}
	public boolean isTxOn() throws Exception
	{
		return xena.PS_ENABLE(_module, _port, _sid) && xena.P_TRAFFIC(_module, _port);
	}
	
	public void report(Reporter report) {
		try {
			int size = getPacketSizeFixed();
			int rate= getRateMbps();
			XenaStatistics RxStat = getRxStatistics();
			XenaStatistics TxStat = getTxStatistics();
			report.report(String.format("Port %d Stream (%d) %s (%s) Packet size %d, rate %d Mbps", _port,_sid,getName(),isTxOn() ? "Running":"Stopped",size,rate));
			report.report(String.format("Tx:%s Rx:%s",TxStat.toString(),RxStat.toString()));
			} catch (Exception e) {
			report.report(e.toString(),e);
			report.report(String.format("Port %d Stream (%d) %s", _port,_sid,getName()));
		}

	}

	public void reportConfiguration(Reporter report) {
		try {
			int size = getPacketSizeFixed();
			int rate= getRateMbps();
			report.report(String.format("Stream ID %d (m%d/p%d): Rate=%d Mbps PacketSize=%d", getSid(),_module, _port,rate,size));
			} catch (Exception e) {
			report.report(e.toString(),e);
			report.report(String.format("Port %d Stream (%d) %s", _port,_sid,getName()));
		}

	}
	public int getPort() {
		return _port;
	}

	public int getIndex() {
		return _sid;
	}

	public String getName() {
		return packet.getName();
	}

	public EthernetPacket getPacket() {
				return packet;
	}
	
	/*
	 * Packet types: FIXED (all packets have min size) INCREMENTING
	 * (incrementing from min to max) BUTTERFLY (min, max, min+1, max?1, min+2,
	 * max?2, etc) RANDOM (random between min and max) MIX (a mixture of sizes
	 * between 56 and 1518, average 464 bytes)
	 */
	public void setPacketSizeFixed(int size) throws Exception {
		xena.PS_PACKETLENGTH(_module, _port, _sid, "RANDOM", size, size);
	}

	public int getPacketSizeFixed() throws Exception {
		return getMinPacketSize();
	}

	public int getMinPacketSize() throws Exception {
		String ret = xena.PS_PACKETLENGTH(_module, _port, _sid);
		String parsed[] = ret.split(" ");
		return Integer.parseInt(parsed[1]);
	}

	public int getMaxPacketSize() throws Exception {
		String ret = xena.PS_PACKETLENGTH(_module, _port, _sid);
		String parsed[] = ret.split(" ");
		return Integer.parseInt(parsed[2]);
	}

	public String getTypePacketSize() throws Exception {
		String ret = xena.PS_PACKETLENGTH(_module, _port, _sid);
		String parsed[] = ret.split(" ");
		return parsed[0];
	}

	public void setPacketFixedSize(int size) throws Exception {
		xena.PS_PACKETLENGTH(_module, _port, _sid, "FIXED", size, size);
	}

	public void setPacketSizeRandom(int min, int max) throws Exception {
		xena.PS_PACKETLENGTH(_module, _port, _sid, "RANDOM", min, max);
	}

	public void setPacketSizeIncrementing(int min, int max) throws Exception {
		xena.PS_PACKETLENGTH(_module, _port, _sid, "INCREMENTING", min, max);
	}

	public void setPacketSizeMix() throws Exception {
		xena.PS_PACKETLENGTH(_module, _port, _sid, "INCREMENTING", 64, 1512);
	}

	public void enable(boolean enable) throws Exception {
		xena.PS_ENABLE(_module, _port, _sid, enable);
	}

	public void setRateFps(int fps) throws Exception {
		xena.PS_RATEPPS(_module, _port, _sid, fps);
	}

	public int getRateFps() throws Exception {
		String ret = xena.PS_RATEPPS(_module, _port, _sid);
		return Integer.parseInt(ret);
	}
	
	public int getRateMbps() throws Exception {
		String ret = xena.PS_RATEL2BPS(_module, _port, _sid);
		long rate = Long.parseLong(ret); 
		if (rate==0)
		{
			ret = xena.PS_RATEPPS(_module, _port, _sid);
			rate = Long.parseLong(ret) * getPacketSizeFixed() ; 
		}
		return (int)( rate/1000000);
	}	
	public void setRatePercent(double rate) throws Exception {
		xena.PS_RATEFRACTION(_module, _port, _sid, 10000*rate);
	}

	public void setRateMbps(double rate) throws Exception {
		setRateL2Bps(rate*1000000);
	}
	
	public void setRateL2Bps(double rate) throws Exception {
		xena.PS_RATEL2BPS(_module, _port, _sid, rate);
	}

	public void setCheckSumError() throws Exception{
		xena.PS_INJECTFCSERR(_module, _port, _sid);
		numberOfInjectedFcsErrors++;
	}
	public void setTPLDError() throws Exception {
		xena.PS_INJECTTPLDERR(_module, _port, _sid);
		setNumberOfInjectedTPLDErrors(getNumberOfInjectedTPLDErrors() + 1);
	}
	//------------------------------------end-------//
	public XenaStatistics getTxStatistics() throws Exception {
		return xena.PT_STREAM(_module, _port, _sid);
	}

	public XenaStatistics getRxStatistics() throws Exception {
		return manager.getRxStatistics(_sid);

	}

	public XenaLatencyStatistics getLatencyStatistics() throws Exception {
		return manager.getRxLatencyStatistics(_sid);
	}

	public long getLostPackets() throws Exception {
		XenaStatistics tx = getTxStatistics();
		XenaStatistics rx = getRxStatistics();
		return tx.packets - rx.packets;

	}
	/*
	 * _module  -(Integer) The module number on xena
	 * _port   - (Integer)The port number   on xena
	 * _sid    - (Integer)The sub-index value of the stream definition
	 * @ by Gideon Gur Arie
	 */
	protected void setPayLoadPrbs( )throws Exception {
		
		xena.PS_PAYLOAD(_module, _port, _sid);
		
	}

	public void apply() throws Exception {
		System.out.print(packet.toString());
		xena.PS_HEADERPROTOCOL(_module, _port, _sid, packet
				.getLayersXenaString());
		xena.PS_PACKETHEADER(_module, _port, _sid, packet.encode());
		xena.PS_BURST(_module, _port, _sid, burst.getSize(), burst.getDensity());
		ArrayList<headerModifier> streamModifiers = packet.getModifiersList();
		if(streamModifiers.size() > _maxNumOfModifiers) {
			throw new Exception(String.format("Number of stream modifiers is %d - exceeds maximal number of allowed modifiers, %d!", streamModifiers.size(), _maxNumOfModifiers));
		}
		//XenaPacket thePacket = new XenaPacket();
		
		if (streamModifiers != null) {
			xena.PS_MODIFIERCOUNT(_module, _port, _sid, streamModifiers.size());
			for (int iterator = 0; iterator < streamModifiers.size(); iterator++) {
				headerModifier	singleModifier = streamModifiers.get(iterator);
				
				xena.PS_MODIFIER(_module, _port, iterator, _sid, singleModifier.getFieldOffset(), singleModifier.getMask(), singleModifier.action.getString(), singleModifier.getRepetition());
				xena.PS_MODIFIERRANGE( _module,_port, iterator,_sid,singleModifier.valueStart,singleModifier.valueStep, singleModifier.valueStop); 
			}
		}

	}
	
	public void setDestenationMacAddress(long value)
	{
		 macaddress.setDestenationMacAddress(value);
	}
	
	public void setSourceMacAddress(long value)
	{
		macaddress.setSourceMacAddress(value);
	}

	public streamRate getRate() {
		return rate;
	}

	public void setRate(streamRate rate) throws Exception {
		this.rate = rate;
		switch (rate.type) {
		case Mbps:
			setRateMbps(rate.value);
			break;
		case Percent:
			setRatePercent(rate.value);
			break;
		case Pps:		
			setRateFps((int)rate.value);
			break;
		default:
			break;
		}    	
    }
	
	public void setRate(streamRateTypes _type, int _value) throws Exception {
		streamRate newRate = new streamRate(_type, _value);
		setRate(newRate);		
	}
	

	public streamBurstParams getBurst(){
		return burst;
	}
	public void setBurst(streamBurstParams burst){
		this.burst = burst;
		
	}
	public void setState(streamStates state) throws Exception{
		this.state = state;
		boolean stateOnOff;
		
		if (state == streamStates.Enabled)
			stateOnOff = true;
		else
			stateOnOff = false;
		
		xena.PS_ENABLE(_module, _port, _sid, stateOnOff);
	}
	
	public streamStates getState(){
		return state;
	}
	/**
	 * @return the numberOfInjectedFcsErrors
	 */
	public int getNumberOfInjectedFcsErrors() {
		return numberOfInjectedFcsErrors;
	}
	public int getNumberOfInjectedTPLDErrors() {
		return numberOfInjectedTPLDErrors;
	}
	public void setNumberOfInjectedTPLDErrors(int numberOfInjectedTPLDErrors) {
		this.numberOfInjectedTPLDErrors = numberOfInjectedTPLDErrors;
	}
	/**
	 * @return the _module
	 */
	public int get_module() {
		return _module;
	}
	/**
	 * @param _module the _module to set
	 */
	public void set_module(int _module) {
		this._module = _module;
	}
	/**
	 * @return the _port
	 */
	public int get_port() {
		return _port;
	}
	/**
	 * @param _port the _port to set
	 */
	public void set_port(int _port) {
		this._port = _port;
	}
}
